@@ -10,7 +10,6 @@
 
 from __future__ import absolute_import
 
-import locale
 import os
 import re
 import signal
@@ -21,9 +20,11 @@ from .pycompat import (
     open,
 )
 from . import (
+    diffhelper,
     encoding,
     error,
     patch as patchmod,
+    pycompat,
     scmutil,
     util,
 )
@@ -62,16 +63,8 @@ try:
     import curses.ascii
 
     curses.error
-except ImportError:
-    # I have no idea if wcurses works with crecord...
-    try:
-        import wcurses as curses
-
-        curses.error
-    except ImportError:
-        # wcurses is not shipped on Windows by default, or python is not
-        # compiled with curses
-        curses = False
+except (ImportError, AttributeError):
+    curses = False
 
 
 class fallbackerror(error.Abort):
@@ -102,7 +95,7 @@ class patchnode(object):
         raise NotImplementedError(b"method must be implemented by subclass")
 
     def allchildren(self):
-        b"Return a list of all of the direct children of this node"
+        """Return a list of all of the direct children of this node"""
         raise NotImplementedError(b"method must be implemented by subclass")
 
     def nextsibling(self):
@@ -264,21 +257,23 @@ class uiheader(patchnode):
         return None
 
     def firstchild(self):
-        b"return the first child of this item, if one exists.  otherwise None."
+        """return the first child of this item, if one exists.  otherwise
+        None."""
         if len(self.hunks) > 0:
             return self.hunks[0]
         else:
             return None
 
     def lastchild(self):
-        b"return the last child of this item, if one exists.  otherwise None."
+        """return the last child of this item, if one exists.  otherwise
+        None."""
         if len(self.hunks) > 0:
             return self.hunks[-1]
         else:
             return None
 
     def allchildren(self):
-        b"return a list of all of the direct children of this node"
+        """return a list of all of the direct children of this node"""
         return self.hunks
 
     def __getattr__(self, name):
@@ -286,7 +281,7 @@ class uiheader(patchnode):
 
 
 class uihunkline(patchnode):
-    b"represents a changed line in a hunk"
+    """represents a changed line in a hunk"""
 
     def __init__(self, linetext, hunk):
         self.linetext = linetext
@@ -319,16 +314,18 @@ class uihunkline(patchnode):
             return None
 
     def parentitem(self):
-        b"return the parent to the current item"
+        """return the parent to the current item"""
         return self.hunk
 
     def firstchild(self):
-        b"return the first child of this item, if one exists.  otherwise None."
+        """return the first child of this item, if one exists.  otherwise
+        None."""
         # hunk-lines don't have children
         return None
 
     def lastchild(self):
-        b"return the last child of this item, if one exists.  otherwise None."
+        """return the last child of this item, if one exists.  otherwise
+        None."""
         # hunk-lines don't have children
         return None
 
@@ -372,25 +369,27 @@ class uihunk(patchnode):
             return None
 
     def parentitem(self):
-        b"return the parent to the current item"
+        """return the parent to the current item"""
         return self.header
 
     def firstchild(self):
-        b"return the first child of this item, if one exists.  otherwise None."
+        """return the first child of this item, if one exists.  otherwise
+        None."""
         if len(self.changedlines) > 0:
             return self.changedlines[0]
         else:
             return None
 
     def lastchild(self):
-        b"return the last child of this item, if one exists.  otherwise None."
+        """return the last child of this item, if one exists.  otherwise
+        None."""
         if len(self.changedlines) > 0:
             return self.changedlines[-1]
         else:
             return None
 
     def allchildren(self):
-        b"return a list of all of the direct children of this node"
+        """return a list of all of the direct children of this node"""
         return self.changedlines
 
     def countchanges(self):
@@ -418,7 +417,7 @@ class uihunk(patchnode):
         contextlen = (
             len(self.before) + len(self.after) + removedconvertedtocontext
         )
-        if self.after and self.after[-1] == b'\\ No newline at end of file\n':
+        if self.after and self.after[-1] == diffhelper.MISSING_NEWLINE_MARKER:
             contextlen -= 1
         fromlen = contextlen + self.removed
         tolen = contextlen + self.added
@@ -502,8 +501,12 @@ class uihunk(patchnode):
         """
         dels = []
         adds = []
+        noeol = False
         for line in self.changedlines:
             text = line.linetext
+            if line.linetext == diffhelper.MISSING_NEWLINE_MARKER:
+                noeol = True
+                break
             if line.applied:
                 if text.startswith(b'+'):
                     dels.append(text[1:])
@@ -513,6 +516,9 @@ class uihunk(patchnode):
                 dels.append(text[1:])
                 adds.append(text[1:])
         hunk = [b'-%s' % l for l in dels] + [b'+%s' % l for l in adds]
+        if noeol and hunk:
+            # Remove the newline from the end of the hunk.
+            hunk[-1] = hunk[-1][:-1]
         h = self._hunk
         return patchmod.recordhunk(
             h.header, h.toline, h.fromline, h.proc, h.before, hunk, h.after
@@ -522,7 +528,7 @@ class uihunk(patchnode):
         return getattr(self._hunk, name)
 
     def __repr__(self):
-        return r'<hunk %r@%d>' % (self.filename(), self.fromline)
+        return '<hunk %r@%d>' % (self.filename(), self.fromline)
 
 
 def filterpatch(ui, chunks, chunkselector, operation=None):
@@ -567,14 +573,12 @@ def chunkselector(ui, headerlist, operation=None):
     """
     ui.write(_(b'starting interactive selection\n'))
     chunkselector = curseschunkselector(headerlist, ui, operation)
-    # This is required for ncurses to display non-ASCII characters in
-    # default user locale encoding correctly.  --immerrr
-    locale.setlocale(locale.LC_ALL, r'')
     origsigtstp = sentinel = object()
     if util.safehasattr(signal, b'SIGTSTP'):
         origsigtstp = signal.getsignal(signal.SIGTSTP)
     try:
-        curses.wrapper(chunkselector.main)
+        with util.with_lc_ctype():
+            curses.wrapper(chunkselector.main)
         if chunkselector.initexc is not None:
             raise chunkselector.initexc
         # ncurses does not restore signal handler for SIGTSTP
@@ -853,7 +857,7 @@ class curseschunkselector(object):
         self.currentselecteditem = currentitem
 
     def updatescroll(self):
-        b"scroll the screen to fully show the currently-selected"
+        """scroll the screen to fully show the currently-selected"""
         selstart = self.selecteditemstartline
         selend = self.selecteditemendline
 
@@ -871,7 +875,7 @@ class curseschunkselector(object):
             self.scrolllines(selstart - padstartbuffered)
 
     def scrolllines(self, numlines):
-        b"scroll the screen up (down) by numlines when numlines >0 (<0)."
+        """scroll the screen up (down) by numlines when numlines >0 (<0)."""
         self.firstlineofpadtoprint += numlines
         if self.firstlineofpadtoprint < 0:
             self.firstlineofpadtoprint = 0
@@ -973,7 +977,7 @@ class curseschunkselector(object):
                 )
 
     def toggleall(self):
-        b"toggle the applied flag of all items."
+        """toggle the applied flag of all items."""
         if self.waslasttoggleallapplied:  # then unapply them this time
             for item in self.headerlist:
                 if item.applied:
@@ -984,8 +988,19 @@ class curseschunkselector(object):
                     self.toggleapply(item)
         self.waslasttoggleallapplied = not self.waslasttoggleallapplied
 
+    def flipselections(self):
+        """
+        Flip all selections. Every selected line is unselected and vice
+        versa.
+        """
+        for header in self.headerlist:
+            for hunk in header.allchildren():
+                for line in hunk.allchildren():
+                    self.toggleapply(line)
+
     def toggleallbetween(self):
-        b"toggle applied on or off for all items in range [lastapplied,current]."
+        """toggle applied on or off for all items in range [lastapplied,
+        current]. """
         if (
             not self.lastapplieditem
             or self.currentselecteditem == self.lastapplieditem
@@ -1026,7 +1041,8 @@ class curseschunkselector(object):
             nextitem = nextitem.nextitem()
 
     def togglefolded(self, item=None, foldparent=False):
-        b"toggle folded flag of specified item (defaults to currently selected)"
+        """toggle folded flag of specified item (defaults to currently
+        selected)"""
         if item is None:
             item = self.currentselecteditem
         if foldparent or (isinstance(item, uiheader) and item.neverunfolded):
@@ -1095,7 +1111,7 @@ class curseschunkselector(object):
         # strip \n, and convert control characters to ^[char] representation
         text = re.sub(
             br'[\x00-\x08\x0a-\x1f]',
-            lambda m: b'^' + chr(ord(m.group()) + 64),
+            lambda m: b'^' + pycompat.sysbytes(chr(ord(m.group()) + 64)),
             text.strip(b'\n'),
         )
 
@@ -1320,7 +1336,7 @@ class curseschunkselector(object):
     def printhunklinesbefore(
         self, hunk, selected=False, towin=True, ignorefolding=False
     ):
-        b"includes start/end line indicator"
+        """includes start/end line indicator"""
         outstr = b""
         # where hunk is in list of siblings
         hunkindex = hunk.header.hunks.index(hunk)
@@ -1529,7 +1545,7 @@ class curseschunkselector(object):
         return numlines
 
     def sigwinchhandler(self, n, frame):
-        b"handle window resizing"
+        """handle window resizing"""
         try:
             curses.endwin()
             self.xscreensize, self.yscreensize = scmutil.termsize(self.ui)
@@ -1599,20 +1615,21 @@ class curseschunkselector(object):
         return colorpair
 
     def initcolorpair(self, *args, **kwargs):
-        b"same as getcolorpair."
+        """same as getcolorpair."""
         self.getcolorpair(*args, **kwargs)
 
     def helpwindow(self):
-        b"print a help window to the screen.  exit after any keypress."
+        """print a help window to the screen.  exit after any keypress."""
         helptext = _(
             """            [press any key to return to the patch-display]
 
-crecord allows you to interactively choose among the changes you have made,
-and confirm only those changes you select for further processing by the command
-you are running (commit/shelve/revert), after confirming the selected
-changes, the unselected changes are still present in your working copy, so you
-can use crecord multiple times to split large changes into smaller changesets.
-the following are valid keystrokes:
+The curses hunk selector allows you to interactively choose among the
+changes you have made, and confirm only those changes you select for
+further processing by the command you are running (such as commit,
+shelve, or revert). After confirming the selected changes, the
+unselected changes are still present in your working copy, so you can
+use the hunk selector multiple times to split large changes into
+smaller changesets. the following are valid keystrokes:
 
               x [space] : (un-)select item ([~]/[x] = partly/fully applied)
                 [enter] : (un-)select item and go to next item of same type
@@ -1629,7 +1646,7 @@ the following are valid keystrokes:
                  ctrl-l : scroll the selected line to the top of the screen
                       m : edit / resume editing the commit message
                       e : edit the currently selected hunk
-                      a : toggle amend mode, only with commit -i
+                      a : toggle all selections
                       c : confirm selected changes
                       r : review/edit and confirm selected changes
                       q : quit without confirming (no changes will be made)
@@ -1654,7 +1671,7 @@ the following are valid keystrokes:
             pass
 
     def commitMessageWindow(self):
-        b"Create a temporary commit message editing window on the screen."
+        """Create a temporary commit message editing window on the screen."""
 
         curses.raw()
         curses.def_prog_mode()
@@ -1704,7 +1721,8 @@ the following are valid keystrokes:
         self.recenterdisplayedarea()
 
     def confirmationwindow(self, windowtext):
-        b"display an informational window, then wait for and return a keypress."
+        """display an informational window, then wait for and return a
+        keypress."""
 
         confirmwin = curses.newwin(self.yscreensize, 0, 0, 0)
         try:
@@ -1747,32 +1765,6 @@ are you sure you want to review/edit and confirm the selected changes [yn]?
         else:
             return False
 
-    def toggleamend(self, opts, test):
-        """Toggle the amend flag.
-
-        When the amend flag is set, a commit will modify the most recently
-        committed changeset, instead of creating a new changeset.  Otherwise, a
-        new changeset will be created (the normal commit behavior).
-        """
-
-        if opts.get(b'amend') is None:
-            opts[b'amend'] = True
-            msg = _(
-                b"Amend option is turned on -- committing the currently "
-                b"selected changes will not create a new changeset, but "
-                b"instead update the most recently committed changeset.\n\n"
-                b"Press any key to continue."
-            )
-        elif opts.get(b'amend') is True:
-            opts[b'amend'] = None
-            msg = _(
-                b"Amend option is turned off -- committing the currently "
-                b"selected changes will create a new changeset.\n\n"
-                b"Press any key to continue."
-            )
-        if not test:
-            self.confirmationwindow(msg)
-
     def recenterdisplayedarea(self):
         """
         once we scrolled with pg up pg down we can be pointing outside of the
@@ -1904,7 +1896,7 @@ are you sure you want to review/edit and confirm the selected changes [yn]?
         elif keypressed in ["q"]:
             raise error.Abort(_(b'user quit'))
         elif keypressed in ['a']:
-            self.toggleamend(self.opts, test)
+            self.flipselections()
         elif keypressed in ["c"]:
             return True
         elif keypressed in ["r"]:
