@@ -92,15 +92,14 @@ import sys
 import threading
 
 from .i18n import _
+from .pycompat import getattr
 from . import (
     node,
     pycompat,
     urllibcompat,
     util,
 )
-from .utils import (
-    procutil,
-)
+from .utils import procutil
 
 httplib = util.httplib
 urlerr = util.urlerr
@@ -108,16 +107,18 @@ urlreq = util.urlreq
 
 DEBUG = None
 
+
 class ConnectionManager(object):
     """
     The connection manager must be able to:
       * keep track of all existing
       """
+
     def __init__(self):
         self._lock = threading.Lock()
-        self._hostmap = collections.defaultdict(list) # host -> [connection]
-        self._connmap = {} # map connections to host
-        self._readymap = {} # map connection to ready state
+        self._hostmap = collections.defaultdict(list)  # host -> [connection]
+        self._connmap = {}  # map connections to host
+        self._readymap = {}  # map connection to ready state
 
     def add(self, host, connection, ready):
         self._lock.acquire()
@@ -169,6 +170,7 @@ class ConnectionManager(object):
         else:
             return dict(self._hostmap)
 
+
 class KeepAliveHandler(object):
     def __init__(self, timeout=None):
         self._cm = ConnectionManager()
@@ -192,7 +194,7 @@ class KeepAliveHandler(object):
 
     def close_all(self):
         """close all open connections"""
-        for host, conns in self._cm.get_all().iteritems():
+        for host, conns in pycompat.iteritems(self._cm.get_all()):
             for h in conns:
                 self._cm.remove(h)
                 h.close()
@@ -214,7 +216,7 @@ class KeepAliveHandler(object):
     def do_open(self, http_class, req):
         host = urllibcompat.gethost(req)
         if not host:
-            raise urlerr.urlerror('no host given')
+            raise urlerr.urlerror(b'no host given')
 
         try:
             h = self._cm.get_ready_conn(host)
@@ -235,8 +237,9 @@ class KeepAliveHandler(object):
                 # no (working) free connections were found.  Create a new one.
                 h = http_class(host, timeout=self._timeout)
                 if DEBUG:
-                    DEBUG.info("creating new connection to %s (%d)",
-                               host, id(h))
+                    DEBUG.info(
+                        b"creating new connection to %s (%d)", host, id(h)
+                    )
                 self._cm.add(host, h, False)
                 self._start_transaction(h, req)
                 r = h.getresponse()
@@ -244,18 +247,19 @@ class KeepAliveHandler(object):
         # to make the error message slightly more useful.
         except httplib.BadStatusLine as err:
             raise urlerr.urlerror(
-                _('bad HTTP status line: %s') % pycompat.sysbytes(err.line))
+                _(b'bad HTTP status line: %s') % pycompat.sysbytes(err.line)
+            )
         except (socket.error, httplib.HTTPException) as err:
             raise urlerr.urlerror(err)
 
         # If not a persistent connection, don't try to reuse it. Look
         # for this using getattr() since vcr doesn't define this
         # attribute, and in that case always close the connection.
-        if getattr(r, r'will_close', True):
+        if getattr(r, 'will_close', True):
             self._cm.remove(h)
 
         if DEBUG:
-            DEBUG.info("STATUS: %s, %s", r.status, r.reason)
+            DEBUG.info(b"STATUS: %s, %s", r.status, r.reason)
         r._handler = self
         r._host = host
         r._url = req.get_full_url()
@@ -280,7 +284,7 @@ class KeepAliveHandler(object):
             # worked.  We'll check the version below, too.
         except (socket.error, httplib.HTTPException):
             r = None
-        except: # re-raises
+        except:  # re-raises
             # adding this block just in case we've missed
             # something we will still raise the exception, but
             # lets try and close the connection and remove it
@@ -291,8 +295,11 @@ class KeepAliveHandler(object):
             # that it's now possible this call will raise
             # a DIFFERENT exception
             if DEBUG:
-                DEBUG.error("unexpected exception - closing "
-                            "connection to %s (%d)", host, id(h))
+                DEBUG.error(
+                    b"unexpected exception - closing connection to %s (%d)",
+                    host,
+                    id(h),
+                )
             self._cm.remove(h)
             h.close()
             raise
@@ -303,12 +310,13 @@ class KeepAliveHandler(object):
             # the socket has been closed by the server since we
             # last used the connection.
             if DEBUG:
-                DEBUG.info("failed to re-use connection to %s (%d)",
-                           host, id(h))
+                DEBUG.info(
+                    b"failed to re-use connection to %s (%d)", host, id(h)
+                )
             r = None
         else:
             if DEBUG:
-                DEBUG.info("re-using connection to %s (%d)", host, id(h))
+                DEBUG.info(b"re-using connection to %s (%d)", host, id(h))
 
         return r
 
@@ -323,24 +331,29 @@ class KeepAliveHandler(object):
         headers.update(sorted(req.unredirected_hdrs.items()))
         headers = util.sortdict((n.lower(), v) for n, v in headers.items())
         skipheaders = {}
-        for n in (r'host', r'accept-encoding'):
+        for n in ('host', 'accept-encoding'):
             if n in headers:
-                skipheaders[r'skip_' + n.replace(r'-', r'_')] = 1
+                skipheaders['skip_' + n.replace('-', '_')] = 1
         try:
             if urllibcompat.hasdata(req):
                 data = urllibcompat.getdata(req)
                 h.putrequest(
-                    req.get_method(), urllibcompat.getselector(req),
-                    **skipheaders)
-                if r'content-type' not in headers:
-                    h.putheader(r'Content-type',
-                                r'application/x-www-form-urlencoded')
-                if r'content-length' not in headers:
-                    h.putheader(r'Content-length', r'%d' % len(data))
+                    req.get_method(),
+                    urllibcompat.getselector(req),
+                    **skipheaders
+                )
+                if 'content-type' not in headers:
+                    h.putheader(
+                        'Content-type', 'application/x-www-form-urlencoded'
+                    )
+                if 'content-length' not in headers:
+                    h.putheader('Content-length', '%d' % len(data))
             else:
                 h.putrequest(
-                    req.get_method(), urllibcompat.getselector(req),
-                    **skipheaders)
+                    req.get_method(),
+                    urllibcompat.getselector(req),
+                    **skipheaders
+                )
         except socket.error as err:
             raise urlerr.urlerror(err)
         for k, v in headers.items():
@@ -356,13 +369,16 @@ class KeepAliveHandler(object):
         try:
             self.parent.requestscount += 1
             self.parent.sentbytescount += (
-                getattr(h, 'sentbytescount', 0) - oldbytescount)
+                getattr(h, 'sentbytescount', 0) - oldbytescount
+            )
         except AttributeError:
             pass
 
+
 class HTTPHandler(KeepAliveHandler, urlreq.httphandler):
     pass
 
+
 class HTTPResponse(httplib.HTTPResponse):
     # we need to subclass HTTPResponse in order to
     # 1) add readline(), readlines(), and readinto() methods
@@ -382,23 +398,23 @@ class HTTPResponse(httplib.HTTPResponse):
     # Both readline and readlines have been stolen with almost no
     # modification from socket.py
 
-
     def __init__(self, sock, debuglevel=0, strict=0, method=None):
         extrakw = {}
         if not pycompat.ispy3:
-            extrakw[r'strict'] = True
-            extrakw[r'buffering'] = True
-        httplib.HTTPResponse.__init__(self, sock, debuglevel=debuglevel,
-                                      method=method, **extrakw)
+            extrakw['strict'] = True
+            extrakw['buffering'] = True
+        httplib.HTTPResponse.__init__(
+            self, sock, debuglevel=debuglevel, method=method, **extrakw
+        )
         self.fileno = sock.fileno
         self.code = None
         self.receivedbytescount = 0
-        self._rbuf = ''
+        self._rbuf = b''
         self._rbufsize = 8096
-        self._handler = None # inserted by the handler later
-        self._host = None    # (same)
-        self._url = None     # (same)
-        self._connection = None # (same)
+        self._handler = None  # inserted by the handler later
+        self._host = None  # (same)
+        self._url = None  # (same)
+        self._connection = None  # (same)
 
     _raw_read = httplib.HTTPResponse.read
     _raw_readinto = getattr(httplib.HTTPResponse, 'readinto', None)
@@ -413,8 +429,9 @@ class HTTPResponse(httplib.HTTPResponse):
             self.fp.close()
             self.fp = None
             if self._handler:
-                self._handler._request_closed(self, self._host,
-                                              self._connection)
+                self._handler._request_closed(
+                    self, self._host, self._connection
+                )
 
     def _close_conn(self):
         self.close()
@@ -444,7 +461,7 @@ class HTTPResponse(httplib.HTTPResponse):
         # implemented using readinto(), which can duplicate self._rbuf
         # if it's not empty.
         s = self._rbuf
-        self._rbuf = ''
+        self._rbuf = b''
         data = self._raw_read(amt)
 
         self.receivedbytescount += len(data)
@@ -468,16 +485,16 @@ class HTTPResponse(httplib.HTTPResponse):
         while True:
             if chunk_left is None:
                 line = self.fp.readline()
-                i = line.find(';')
+                i = line.find(b';')
                 if i >= 0:
-                    line = line[:i] # strip chunk-extensions
+                    line = line[:i]  # strip chunk-extensions
                 try:
                     chunk_left = int(line, 16)
                 except ValueError:
                     # close the connection as protocol synchronization is
                     # probably lost
                     self.close()
-                    raise httplib.IncompleteRead(''.join(parts))
+                    raise httplib.IncompleteRead(b''.join(parts))
                 if chunk_left == 0:
                     break
             if amt is None:
@@ -485,18 +502,18 @@ class HTTPResponse(httplib.HTTPResponse):
             elif amt < chunk_left:
                 parts.append(self._safe_read(amt))
                 self.chunk_left = chunk_left - amt
-                return ''.join(parts)
+                return b''.join(parts)
             elif amt == chunk_left:
                 parts.append(self._safe_read(amt))
                 self._safe_read(2)  # toss the CRLF at the end of the chunk
                 self.chunk_left = None
-                return ''.join(parts)
+                return b''.join(parts)
             else:
                 parts.append(self._safe_read(chunk_left))
                 amt -= chunk_left
 
             # we read the whole chunk, get another
-            self._safe_read(2)      # toss the CRLF at the end of the chunk
+            self._safe_read(2)  # toss the CRLF at the end of the chunk
             chunk_left = None
 
         # read and discard trailer up to the CRLF terminator
@@ -507,17 +524,17 @@ class HTTPResponse(httplib.HTTPResponse):
                 # a vanishingly small number of sites EOF without
                 # sending the trailer
                 break
-            if line == '\r\n':
+            if line == b'\r\n':
                 break
 
         # we read everything; close the "file"
         self.close()
 
-        return ''.join(parts)
+        return b''.join(parts)
 
     def readline(self):
         # Fast path for a line is already available in read buffer.
-        i = self._rbuf.find('\n')
+        i = self._rbuf.find(b'\n')
         if i >= 0:
             i += 1
             line = self._rbuf[:i]
@@ -525,7 +542,11 @@ class HTTPResponse(httplib.HTTPResponse):
             return line
 
         # No newline in local buffer. Read until we find one.
-        chunks = [self._rbuf]
+        # readinto read via readinto will already return _rbuf
+        if self._raw_readinto is None:
+            chunks = [self._rbuf]
+        else:
+            chunks = []
         i = -1
         readsize = self._rbufsize
         while True:
@@ -541,7 +562,7 @@ class HTTPResponse(httplib.HTTPResponse):
                 pass
 
             chunks.append(new)
-            i = new.find('\n')
+            i = new.find(b'\n')
             if i >= 0:
                 break
 
@@ -549,13 +570,13 @@ class HTTPResponse(httplib.HTTPResponse):
 
         # EOF
         if i == -1:
-            self._rbuf = ''
-            return ''.join(chunks)
+            self._rbuf = b''
+            return b''.join(chunks)
 
         i += 1
         self._rbuf = chunks[-1][i:]
         chunks[-1] = chunks[-1][:i]
-        return ''.join(chunks)
+        return b''.join(chunks)
 
     def readlines(self, sizehint=0):
         total = 0
@@ -575,7 +596,7 @@ class HTTPResponse(httplib.HTTPResponse):
             res = self.read(len(dest))
             if not res:
                 return 0
-            dest[0:len(res)] = res
+            dest[0 : len(res)] = res
             return len(res)
         total = len(dest)
         have = len(self._rbuf)
@@ -595,9 +616,10 @@ class HTTPResponse(httplib.HTTPResponse):
 
         dest[0:have] = self._rbuf
         got += len(self._rbuf)
-        self._rbuf = ''
+        self._rbuf = b''
         return got
 
+
 def safesend(self, str):
     """Send `str' to the server.
 
@@ -625,13 +647,13 @@ def safesend(self, str):
     # NOTE: we DO propagate the error, though, because we cannot simply
     #       ignore the error... the caller will know if they can retry.
     if self.debuglevel > 0:
-        print("send:", repr(str))
+        print(b"send:", repr(str))
     try:
         blocksize = 8192
         read = getattr(str, 'read', None)
         if read is not None:
             if self.debuglevel > 0:
-                print("sending a read()able")
+                print(b"sending a read()able")
             data = read(blocksize)
             while data:
                 self.sock.sendall(data)
@@ -642,7 +664,7 @@ def safesend(self, str):
             self.sentbytescount += len(str)
     except socket.error as v:
         reraise = True
-        if v.args[0] == errno.EPIPE:      # Broken pipe
+        if v.args[0] == errno.EPIPE:  # Broken pipe
             if self._HTTPConnection__state == httplib._CS_REQ_SENT:
                 self._broken_pipe_resp = None
                 self._broken_pipe_resp = self.getresponse()
@@ -651,9 +673,11 @@ def safesend(self, str):
         if reraise:
             raise
 
+
 def wrapgetresponse(cls):
     """Wraps getresponse in cls with a broken-pipe sane version.
     """
+
     def safegetresponse(self):
         # In safesend() we might set the _broken_pipe_resp
         # attribute, in which case the socket has already
@@ -663,9 +687,11 @@ def wrapgetresponse(cls):
         if r is not None:
             return r
         return cls.getresponse(self)
+
     safegetresponse.__doc__ = cls.getresponse.__doc__
     return safegetresponse
 
+
 class HTTPConnection(httplib.HTTPConnection):
     # url.httpsconnection inherits from this. So when adding/removing
     # attributes, be sure to audit httpsconnection() for unintended
@@ -681,6 +707,7 @@ class HTTPConnection(httplib.HTTPConnection):
         self.sentbytescount = 0
         self.receivedbytescount = 0
 
+
 #########################################################################
 #####   TEST FUNCTIONS
 #########################################################################
@@ -688,7 +715,7 @@ class HTTPConnection(httplib.HTTPConnection):
 
 def continuity(url):
     md5 = hashlib.md5
-    format = '%25s: %s'
+    format = b'%25s: %s'
 
     # first fetch the file with the normal http handler
     opener = urlreq.buildopener()
@@ -697,7 +724,7 @@ def continuity(url):
     foo = fo.read()
     fo.close()
     m = md5(foo)
-    print(format % ('normal urllib', node.hex(m.digest())))
+    print(format % (b'normal urllib', node.hex(m.digest())))
 
     # now install the keepalive handler and try again
     opener = urlreq.buildopener(HTTPHandler())
@@ -707,10 +734,10 @@ def continuity(url):
     foo = fo.read()
     fo.close()
     m = md5(foo)
-    print(format % ('keepalive read', node.hex(m.digest())))
+    print(format % (b'keepalive read', node.hex(m.digest())))
 
     fo = urlreq.urlopen(url)
-    foo = ''
+    foo = b''
     while True:
         f = fo.readline()
         if f:
@@ -719,28 +746,31 @@ def continuity(url):
             break
     fo.close()
     m = md5(foo)
-    print(format % ('keepalive readline', node.hex(m.digest())))
+    print(format % (b'keepalive readline', node.hex(m.digest())))
+
 
 def comp(N, url):
-    print('  making %i connections to:\n  %s' % (N, url))
+    print(b'  making %i connections to:\n  %s' % (N, url))
 
-    procutil.stdout.write('  first using the normal urllib handlers')
+    procutil.stdout.write(b'  first using the normal urllib handlers')
     # first use normal opener
     opener = urlreq.buildopener()
     urlreq.installopener(opener)
     t1 = fetch(N, url)
-    print('  TIME: %.3f s' % t1)
+    print(b'  TIME: %.3f s' % t1)
 
-    procutil.stdout.write('  now using the keepalive handler       ')
+    procutil.stdout.write(b'  now using the keepalive handler       ')
     # now install the keepalive handler and try again
     opener = urlreq.buildopener(HTTPHandler())
     urlreq.installopener(opener)
     t2 = fetch(N, url)
-    print('  TIME: %.3f s' % t2)
-    print('  improvement factor: %.2f' % (t1 / t2))
+    print(b'  TIME: %.3f s' % t2)
+    print(b'  improvement factor: %.2f' % (t1 / t2))
+
 
 def fetch(N, url, delay=0):
     import time
+
     lens = []
     starttime = time.time()
     for i in range(N):
@@ -756,61 +786,67 @@ def fetch(N, url, delay=0):
     for i in lens[1:]:
         j = j + 1
         if not i == lens[0]:
-            print("WARNING: inconsistent length on read %i: %i" % (j, i))
+            print(b"WARNING: inconsistent length on read %i: %i" % (j, i))
 
     return diff
 
+
 def test_timeout(url):
     global DEBUG
     dbbackup = DEBUG
+
     class FakeLogger(object):
         def debug(self, msg, *args):
             print(msg % args)
+
         info = warning = error = debug
+
     DEBUG = FakeLogger()
-    print("  fetching the file to establish a connection")
+    print(b"  fetching the file to establish a connection")
     fo = urlreq.urlopen(url)
     data1 = fo.read()
     fo.close()
 
     i = 20
-    print("  waiting %i seconds for the server to close the connection" % i)
+    print(b"  waiting %i seconds for the server to close the connection" % i)
     while i > 0:
-        procutil.stdout.write('\r  %2i' % i)
+        procutil.stdout.write(b'\r  %2i' % i)
         procutil.stdout.flush()
         time.sleep(1)
         i -= 1
-    procutil.stderr.write('\r')
+    procutil.stderr.write(b'\r')
 
-    print("  fetching the file a second time")
+    print(b"  fetching the file a second time")
     fo = urlreq.urlopen(url)
     data2 = fo.read()
     fo.close()
 
     if data1 == data2:
-        print('  data are identical')
+        print(b'  data are identical')
     else:
-        print('  ERROR: DATA DIFFER')
+        print(b'  ERROR: DATA DIFFER')
 
     DEBUG = dbbackup
 
 
 def test(url, N=10):
-    print("performing continuity test (making sure stuff isn't corrupted)")
+    print(b"performing continuity test (making sure stuff isn't corrupted)")
     continuity(url)
-    print('')
-    print("performing speed comparison")
+    print(b'')
+    print(b"performing speed comparison")
     comp(N, url)
-    print('')
-    print("performing dropped-connection check")
+    print(b'')
+    print(b"performing dropped-connection check")
     test_timeout(url)
 
+
 if __name__ == '__main__':
     import time
+
     try:
         N = int(sys.argv[1])
         url = sys.argv[2]
     except (IndexError, ValueError):
-        print("%s <integer> <url>" % sys.argv[0])
+        print(b"%s <integer> <url>" % sys.argv[0])
     else:
         test(url, N)
